import { Component, Event, h, Method, Prop, State, Watch } from '@stencil/core';
import anime from 'animejs';
export class WrAccordion {
  constructor() {
    /**
     * The initial state. This is set to open in case the JS errors and to fire the correct closing event
     * on initial render.
     */
    this.open = true;
    this.sizes = {};
  }
  /**
   * Lifecycle hook to close the component after render. Should the script error before loading the content is still
   * readable.
   */
  async componentDidLoad() {
    const resizeObserver = new ResizeObserver(() => this.calculateSizes());
    await this.calculateSizes();
    this.open = false;
    this.checkBackgroundImage();
    resizeObserver.observe(this.accordion);
  }
  async watchOpen(newValue) {
    if (newValue)
      this.accordionOpened.emit('starting');
    let elementHeight;
    const tl = anime.timeline({
      easing: newValue ? 'easeInCubic' : 'easeOutCubic',
      duration: 300,
    });
    if (newValue) {
      elementHeight = this.sizes.openedSize;
    }
    else {
      elementHeight = this.sizes.closedSize;
    }
    if (newValue)
      this.accordionOpened.emit('animating');
    // Set the accordion to either full height or the header height.
    tl.add({
      targets: this.accordion,
      height: elementHeight,
    });
    if (this.background) {
      // Set the background to either full height or the header height.
      tl.add({
        targets: this.backgroundElement,
        height: elementHeight,
      }, 0);
      // Scale the background to fit.
      tl.add({
        targets: this.backgroundElement,
        scale: newValue ? [1, 1.25] : [1.25, 1],
      }, '+=150');
    }
    // Flow in the content.
    tl.add({
      targets: this.contentElement,
      translateY: newValue ? ['-2rem', 0] : [0, '-2rem'],
      opacity: newValue ? [0, 0, 100] : [100, 0],
    }, this.background ? '-=300' : 0);
    if (newValue)
      this.accordionOpened.emit('processed');
  }
  async closeMe() {
    this.open = false;
  }
  async openMe() {
    this.open = true;
  }
  render() {
    return (h("article", { ref: (el) => this.accordion = el, class: ['accordion', this.open ? 'active' : ''].join(' ') },
      this.renderImage(),
      h("header", { class: 'accordion__header', onClick: () => this.open = !this.open },
        h("h3", { class: 'accordion__title' }, this.accordionTitle),
        this.renderStateButton()),
      h("section", { class: 'accordion__content', ref: (el) => this.contentElement = el, "aria-hidden": this.open ? 'true' : 'false' },
        h("slot", null))));
  }
  renderStateButton() {
    return (h("button", { class: ['accordion__button', this.open ? 'active' : ''].join(' '), name: `${this.open ? 'Verberg' : 'Toon'} de inhoud van ${this.accordionTitle}` },
      h("span", null),
      h("span", null)));
  }
  /**
   * Renders the background image for the accordion if supplied.
   *
   * Has a built-in check for errors to remove itself in prevention of default image error.
   */
  renderImage() {
    if (!this.background) {
      return;
    }
    return (h("figure", { class: ['accordion__background', this.open ? 'active' : ''].join(' '), ref: (el) => this.backgroundElement = el },
      h("img", { src: this.background, alt: '' })));
  }
  /**
   * Recalculate te sizes of the accordion.
   */
  async calculateSizes() {
    const headerHeight = this.accordion.querySelector('.accordion__header').clientHeight + 1; // Add 1 px for the bottom border.
    this.sizes.openedSize = headerHeight + this.contentElement.clientHeight;
    this.sizes.closedSize = headerHeight;
  }
  /**
   * Checks if the accordion background image should exist.
   * @protected
   */
  checkBackgroundImage() {
    if (this.backgroundElement) {
      this.backgroundElement.querySelector('img')
        .addEventListener('error', () => {
        this.backgroundElement.parentElement.removeChild(this.backgroundElement);
        this.background = null;
      });
    }
  }
  static get is() { return "wr-accordion"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["wr-accordion.css"]
  }; }
  static get styleUrls() { return {
    "$": ["wr-accordion.css"]
  }; }
  static get properties() { return {
    "accordionTitle": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": true,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The title for the accordion. This is always shown."
      },
      "attribute": "accordion-title",
      "reflect": false
    },
    "background": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The background image which scales when opening."
      },
      "attribute": "background",
      "reflect": false
    }
  }; }
  static get states() { return {
    "open": {}
  }; }
  static get events() { return [{
      "method": "accordionOpened",
      "name": "accordionOpened",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Fires when the element is opened. The Event.detail property is a string\nset to the current opening state.\n\nEvent fires the following 3 times:\n  - `starting`\n  - `animating`\n  - `processed`"
      },
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "closeMe": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "openMe": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "calculateSizes": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Recalculate te sizes of the accordion.",
        "tags": []
      }
    }
  }; }
  static get watchers() { return [{
      "propName": "open",
      "methodName": "watchOpen"
    }]; }
}
